/*
 * vifctl/vif_notify.c
 * $Rev: 1202 $
 *
 * Copyright (C) 2009-2010 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

/**
 * @file vif_notify.c
 * @brief Functions for sending notification about parameter changes to a
 * IPC message queue
 */

/* INCLUDES *******************************************************************/

#include <netinet/in.h>                 // struct in_addr
#include <linux/if.h>                   // IFNAMSIZ
#include <sys/msg.h>                    // msgget(), msgrecv(), msgsnd(), ...
#include <errno.h>                      // errno
#include <stdio.h>                      // printf()
#include <string.h>                     // memset(), strerror()

extern int (*vif_debug)(const char *format, ...);
extern int (*vif_error)(const char *format, ...);

/* MESSAGE QUEUE TYPES ********************************************************/

#define VIF_NOTIFY_KEY      1337    /**< Name of notification message queue */

/** Message format to indicate a ioctl() */
typedef struct {
    long mtype;                     /**< Message type */
    int ioctl_type;                 /**< ioctl() type */
    char dev_name[IFNAMSIZ + 1];    /**< Virtual interface device name */
} ioctl_msg;

#define IOCTL_MSG_SIZE      sizeof(ioctl_msg)
#define VIF_SIOCSIW         0x10    /**< Message type: ioctl() set wireless parameter */


/* FUNCTIONS ******************************************************************/

/**
 * Send notification message for executed ioctl()
 * @param ioctl_type Ioctl() type (see wireless.h)
 * @param dev_name Virtual interface device name
 * @return 0 on success, -1 else
 */
int
vif_notify_ioctl(int ioctl_type, const char *dev_name)
{
    ioctl_msg msg;
    int write_flags = 0600;

    int msgq_id = msgget(VIF_NOTIFY_KEY, write_flags);
    if (msgq_id < 0)
    {
        vif_debug("vif_notify_ioctl(): Failed to update WlanModel\n");
        return -1;
    }

    // prepare message (fill type and vif name)
    memset(&msg, 0, IOCTL_MSG_SIZE);
    msg.mtype = VIF_SIOCSIW;
    msg.ioctl_type = ioctl_type;
    strncpy(msg.dev_name, dev_name, IFNAMSIZ);
    msg.dev_name[IFNAMSIZ] = '\0';          // make sure we have proper string

    if (msgsnd(msgq_id, &msg, IOCTL_MSG_SIZE, 0) < 0)
    {
        vif_debug("vif_notify_ioctl(): msgsnd (%s)\n", strerror(errno));
        return -1;
    }

    return 0;
}
