/*
 * wlanmodel/PMTypes.h
 *
 * Copyright (C) 2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __PMTYPES_H
#define __PMTYPES_H

/**
 * Header to include PacketModeller related types
 */

/** INCLUDES ******************************************************************/
#include <limits.h>						// for HOST_NAME_MAX
#include <netinet/in.h>
#include <net/ethernet.h>
#include <linux/if_ether.h>
#include <linux/types.h>
#include <linux/wireless.h>


/** IW TYPES ******************************************************************/
#ifndef IWLIB_H

/** If we didn't include iwlib.h before, define iw data types ourselves.
 *  This is used to compile WlanModel without dependency to wireless-tools headers
 *  (taken from /usr/include/iwlib.h) */

/** Some usefull constants */
#define KILO    1e3
#define MEGA    1e6
#define GIGA    1e9

/** Shortcuts */
typedef struct iw_statistics    iwstats;
typedef struct iw_range         iwrange;
typedef struct iw_param         iwparam;
typedef struct iw_freq          iwfreq;
typedef struct iw_quality       iwqual;
typedef struct iw_priv_args     iwprivargs;
typedef struct sockaddr         sockaddr;

/** Structure for storing all wireless information for each device */
typedef struct wireless_config
{
  char      name[IFNAMSIZ + 1];             // Wireless/protocol name
  int       has_nwid;
  iwparam   nwid;                           // Network ID
  int       has_freq;
  double    freq;                           // Frequency/channel
  int       freq_flags;
  int       has_key;
  unsigned char key[IW_ENCODING_TOKEN_MAX]; // Encoding key used
  int       key_size;                       // Number of bytes
  int       key_flags;                      // Various flags
  int       has_essid;
  int       essid_on;
  char      essid[IW_ESSID_MAX_SIZE + 1];   // ESSID (extended network)
  int       has_mode;
  int       mode;                           // Operation mode
} wireless_config;

/** Structure for storing all wireless information for each device */
typedef struct wireless_info
{
  struct wireless_config    b;              // Basic information

  int       has_sens;
  iwparam   sens;                           // Sensitivity
  int       has_nickname;
  char      nickname[IW_ESSID_MAX_SIZE + 1]; // NickName
  int       has_ap_addr;
  sockaddr  ap_addr;                        // Access point address
  int       has_bitrate;
  iwparam   bitrate;                        // Bit rate in bps
  int       has_rts;
  iwparam   rts;                            // RTS threshold in bytes
  int       has_frag;
  iwparam   frag;                           // Fragmentation threshold in bytes
  int       has_power;
  iwparam   power;                          // Power management parameters
  int       has_txpower;
  iwparam   txpower;                        // Transmit Power in dBm
  int       has_retry;
  iwparam   retry;                          // Retry limit or lifetime

  /** Stats */
  iwstats   stats;
  int       has_stats;
  iwrange   range;
  int       has_range;

  /** Auth params for WPA/802.1x/802.11i */
  int       auth_key_mgmt;
  int       has_auth_key_mgmt;
  int       auth_cipher_pairwise;
  int       has_auth_cipher_pairwise;
  int       auth_cipher_group;
  int       has_auth_cipher_group;
} wireless_info;

#endif

/** PM VIF DATA TYPES *********************************************************/

/** Maximum number of virtual interfaces that WlanModel can handle */
#define MAX_NUM_VIF 	2

/** maximal size of an Ethernet frame (MTU 1500) */
#define MAX_FRAME_SIZE          (1500 + sizeof(struct ether_header))

typedef unsigned char mac_addr [ETH_ALEN];
typedef char host_id [2 * ETH_ALEN + 1];

#define max(a, b) ((a) > (b) ? (a) : (b))


/** PM STRUCT DEFINITIONS ****************************************************/

/**
 *  Message header
 *  @var type: message type
 *  @var id: sender identification
 */
typedef struct {
	uint8_t type;
	host_id id;
} msgHeader;

/**
 * Container for RAW Ethernet frame
 */
typedef char rawFrame [MAX_FRAME_SIZE];

/**
 *  Data message (encapsulated wireless packet)
 *  @var pktLength: length of Ethernet packet
 *  @var wlanPkt: raw Ethernet packet
 */
struct s_dataMsg {
    msgHeader header;
    uint16_t vifIndex;
    uint16_t pktLength;
    rawFrame wlanPkt;
} __attribute__((__packed__));      // pack structs to provide padding problems
typedef struct s_dataMsg dataMsg;

/**
 * Vif parameter format for configNotification
 * --> it's a union(!) so always only holds one of the below values
 */
typedef union {
	int16_t					txpower;
	int16_t					retry;
	int16_t                	rts;
	int16_t					frag;
	int16_t                	sensitivity;
    uint16_t				channel;
} vifParameter;

/**
 * Configuration information
 * @var type: parameter type
 */
struct s_configNotification {
	vifParameter value;
	uint16_t vifIndex;
	uint16_t type;
} __attribute__((__packed__));      // pack structs to provide padding problems
typedef struct s_configNotification configNotification;

/**
 *  Configuration message
 */
typedef struct {
    msgHeader header;
    configNotification vifData;
} configMsg;

/**
 * Vif registration information
 */
struct s_vifRegistrationData {
    char            vifName[IFNAMSIZ + 1];
    unsigned char   vifMac[ETH_ALEN];
    int16_t			vifIndex;
    int16_t         txpower;        // [mW]
    int16_t         retry;
    int16_t         rts;            // [Bytes]
    int16_t         frag;           // [Bytes]
    int16_t         sensitivity;    // [dB] (negative)
    uint16_t        channel;
} __attribute__((__packed__));      // pack structs to provide padding problems
typedef struct s_vifRegistrationData vifRegistrationData;

/**
 * Host registration information
 */
struct s_registrationInfo {
    char name[HOST_NAME_MAX];
    struct sockaddr_storage addr;
	uint16_t msgId;
	uint16_t numVif;
    vifRegistrationData vifData[MAX_NUM_VIF];
} __attribute__((__packed__));      // pack structs to provide padding problems
typedef struct s_registrationInfo registrationInfo;

/**
 *  Registration message
 */
typedef struct {
    msgHeader header;
    registrationInfo info;
} registrationMsg;

/**
 *  Plain message (only has id; for ack and de-register)
 *  @var msg_id: message identification
 */
typedef struct {
    msgHeader header;
    uint8_t msgId;
} plainMsg;


/** PM MESSAGE TYPES *********************************************************/

#define DATA_MSG                0
#define DATA_PREAMBLE_SIZE      (sizeof(msgHeader) + 2 * sizeof(uint16_t))
#define DATA_MSG_SIZE           sizeof(dataMsg)

#define CONFIG_MSG              1
#define CONFIG_MSG_SIZE         sizeof(configMsg)

#define REGISTRATION_MSG      	2
#define REGISTRATION_MSG_SIZE   sizeof(registrationMsg)

#define PLAIN_MSG_SIZE          sizeof(plainMsg)

#define ACK_MSG                 3
#define ACK_MSG_SIZE            PLAIN_MSG_SIZE

#define DEREGISTRATION_MSG      4
#define DEREGISTRATION_MSG_SIZE PLAIN_MSG_SIZE

// maximal size of a node <-> model message
#define MAX_MSG_SIZE            max(DATA_MSG_SIZE, REGISTRATION_MSG_SIZE)

#endif // __PMTYPES_H
